#!/usr/bin/env python3
"""
Build and publish script for Task Manager MCP Servers

This script handles the complete build and publish process for PyPI.
"""

import subprocess
import sys
import os
import shutil
from pathlib import Path
import json
import re


def run_command(cmd, description, check=True):
    """Run a command and handle errors"""
    print(f"\n{'='*60}")
    print(f"Running: {description}")
    print(f"Command: {' '.join(cmd)}")
    print(f"{'='*60}")
    
    try:
        result = subprocess.run(cmd, check=check, capture_output=True, text=True)
        if result.stdout:
            print("STDOUT:", result.stdout)
        if result.stderr:
            print("STDERR:", result.stderr)
        
        if check and result.returncode != 0:
            print(f"❌ {description} failed with exit code {result.returncode}")
            return False
        else:
            print(f"✅ {description} completed successfully")
            return True
            
    except subprocess.CalledProcessError as e:
        print(f"❌ {description} failed: {e}")
        if e.stdout:
            print("STDOUT:", e.stdout)
        if e.stderr:
            print("STDERR:", e.stderr)
        return False
    except FileNotFoundError:
        print(f"❌ Command not found: {cmd[0]}")
        return False


def check_prerequisites():
    """Check that all prerequisites are installed"""
    print("🔍 Checking prerequisites...")
    
    required_tools = [
        ("python", "Python interpreter"),
        ("pip", "Python package installer"),
        ("twine", "PyPI upload tool")
    ]
    
    missing_tools = []
    
    for tool, description in required_tools:
        if not shutil.which(tool):
            missing_tools.append((tool, description))
    
    if missing_tools:
        print("❌ Missing required tools:")
        for tool, description in missing_tools:
            print(f"  - {tool}: {description}")
        print("\nPlease install missing tools:")
        print("  pip install twine build")
        return False
    
    print("✅ All prerequisites are available")
    return True


def clean_build_artifacts():
    """Clean previous build artifacts"""
    print("🧹 Cleaning build artifacts...")
    
    artifacts_to_clean = [
        "build",
        "dist", 
        "*.egg-info",
        "__pycache__",
        "htmlcov",
        ".coverage",
        "coverage.xml"
    ]
    
    project_root = Path.cwd()
    
    for pattern in artifacts_to_clean:
        if "*" in pattern:
            # Handle glob patterns
            for path in project_root.glob(f"**/{pattern}"):
                if path.is_dir():
                    shutil.rmtree(path, ignore_errors=True)
                    print(f"  Removed directory: {path}")
                elif path.is_file():
                    path.unlink()
                    print(f"  Removed file: {path}")
        else:
            # Handle direct paths
            path = project_root / pattern
            if path.exists():
                if path.is_dir():
                    shutil.rmtree(path, ignore_errors=True)
                    print(f"  Removed directory: {path}")
                elif path.is_file():
                    path.unlink()
                    print(f"  Removed file: {path}")
    
    print("✅ Build artifacts cleaned")


def run_tests():
    """Run the test suite"""
    print("🧪 Running test suite...")
    
    test_commands = [
        (["python", "-m", "pytest", "--tb=short", "-v"], "Unit and Integration Tests"),
        (["python", "-m", "pytest", "--cov=task_management_mcp", "--cov=task_assignment_mcp", "--cov=shared", "--cov-report=term-missing"], "Coverage Report"),
    ]
    
    for cmd, description in test_commands:
        if not run_command(cmd, description):
            print("❌ Tests failed. Please fix issues before publishing.")
            return False
    
    print("✅ All tests passed")
    return True


def run_quality_checks():
    """Run code quality checks"""
    print("🔍 Running code quality checks...")
    
    quality_commands = [
        (["python", "-m", "black", "--check", "."], "Code Formatting Check"),
        (["python", "-m", "isort", "--check-only", "."], "Import Sorting Check"),
        (["python", "-m", "flake8", "task_management_mcp", "task_assignment_mcp", "shared", "--max-line-length=100"], "Code Style Check"),
    ]
    
    for cmd, description in quality_commands:
        if not run_command(cmd, description, check=False):
            print(f"⚠️ {description} found issues. Consider fixing before publishing.")
    
    print("✅ Quality checks completed")
    return True


def update_version():
    """Update version number"""
    print("📝 Version management...")
    
    # Read current version from pyproject.toml
    pyproject_path = Path("pyproject.toml")
    if not pyproject_path.exists():
        print("❌ pyproject.toml not found")
        return False
    
    with open(pyproject_path, 'r') as f:
        content = f.read()
    
    # Extract current version
    version_match = re.search(r'version = "([^"]+)"', content)
    if not version_match:
        print("❌ Could not find version in pyproject.toml")
        return False
    
    current_version = version_match.group(1)
    print(f"Current version: {current_version}")
    
    # Ask user if they want to update version
    response = input(f"Current version is {current_version}. Update version? (y/N): ").strip().lower()
    
    if response == 'y':
        new_version = input("Enter new version: ").strip()
        if new_version:
            # Update version in pyproject.toml
            new_content = re.sub(r'version = "[^"]+"', f'version = "{new_version}"', content)
            with open(pyproject_path, 'w') as f:
                f.write(new_content)
            
            # Update version in __init__.py files
            init_files = [
                "task_management_mcp/__init__.py",
                "task_assignment_mcp/__init__.py",
                "shared/__init__.py"
            ]
            
            for init_file in init_files:
                init_path = Path(init_file)
                if init_path.exists():
                    with open(init_path, 'r') as f:
                        init_content = f.read()
                    
                    new_init_content = re.sub(r'__version__ = "[^"]+"', f'__version__ = "{new_version}"', init_content)
                    
                    with open(init_path, 'w') as f:
                        f.write(new_init_content)
            
            print(f"✅ Version updated to {new_version}")
        else:
            print("No version provided, keeping current version")
    else:
        print("Keeping current version")
    
    return True


def build_package():
    """Build the package"""
    print("📦 Building package...")
    
    build_commands = [
        (["python", "-m", "build"], "Building source and wheel distributions")
    ]
    
    for cmd, description in build_commands:
        if not run_command(cmd, description):
            return False
    
    # Check that build artifacts were created
    dist_path = Path("dist")
    if not dist_path.exists() or not list(dist_path.glob("*")):
        print("❌ No build artifacts found in dist/")
        return False
    
    print("✅ Package built successfully")
    
    # List build artifacts
    print("\nBuild artifacts:")
    for artifact in dist_path.glob("*"):
        print(f"  - {artifact.name}")
    
    return True


def publish_to_pypi():
    """Publish package to PyPI"""
    print("🚀 Publishing to PyPI...")
    
    # Ask for confirmation
    response = input("Are you sure you want to publish to PyPI? (y/N): ").strip().lower()
    if response != 'y':
        print("Publication cancelled")
        return False
    
    # Check for PyPI credentials
    if not (os.environ.get("TWINE_USERNAME") or os.environ.get("TWINE_PASSWORD")):
        print("⚠️ PyPI credentials not found in environment variables")
        print("You can set them with:")
        print("  export TWINE_USERNAME=your_username")
        print("  export TWINE_PASSWORD=your_password")
        print("Or use API token:")
        print("  export TWINE_USERNAME=__token__")
        print("  export TWINE_PASSWORD=your_api_token")
    
    # Upload to PyPI
    upload_cmd = ["python", "-m", "twine", "upload", "dist/*"]
    
    if not run_command(upload_cmd, "Uploading to PyPI"):
        print("❌ Upload failed. Check credentials and try again.")
        return False
    
    print("✅ Package published successfully to PyPI!")
    return True


def main():
    """Main build and publish workflow"""
    print("🚀 Task Manager MCP Servers - Build and Publish")
    print("=" * 60)
    
    # Change to project root
    script_dir = Path(__file__).parent
    project_root = script_dir.parent
    os.chdir(project_root)
    
    print(f"Working directory: {project_root}")
    
    # Run all steps
    steps = [
        ("Prerequisites Check", check_prerequisites),
        ("Clean Build Artifacts", clean_build_artifacts),
        ("Run Tests", run_tests),
        ("Quality Checks", run_quality_checks),
        ("Version Management", update_version),
        ("Build Package", build_package),
    ]
    
    for step_name, step_func in steps:
        print(f"\n{'='*60}")
        print(f"Step: {step_name}")
        print(f"{'='*60}")
        
        if not step_func():
            print(f"❌ Step '{step_name}' failed. Stopping build process.")
            sys.exit(1)
    
    # Ask about publishing
    print(f"\n{'='*60}")
    print("Build completed successfully!")
    print(f"{'='*60}")
    
    response = input("Do you want to publish to PyPI now? (y/N): ").strip().lower()
    if response == 'y':
        if publish_to_pypi():
            print("\n🎉 Build and publish completed successfully!")
        else:
            print("\n💥 Publish failed, but build was successful.")
            sys.exit(1)
    else:
        print("\nBuild completed. You can publish later with:")
        print("  python -m twine upload dist/*")
    
    print("\nNext steps:")
    print("1. Test the published package: pip install task-manager-mcp")
    print("2. Update documentation if needed")
    print("3. Create a GitHub release")
    print("4. Announce the release")


if __name__ == "__main__":
    main()
