"""
Task Generation Agent for Task Manager MCP Servers

Adapted from the original project's TaskGenerationAgent for use in MCP servers.
This agent converts requirements documents into structured task lists.
"""

import os
import json
import requests
from typing import Dict, List, Optional


class TaskGenerationAgent:
    """
    任务生成Agent - 负责将用户的需求解析成高阶的可执行任务列表。
    """

    SYSTEM_PROMPT_TEMPLATE = """
You are a Chief Product Officer (CPO) AI. Your sole responsibility is to break down a user's project request into 3 to 5 high-level, strategic "Epics" or "Features".

**CRITICAL RULES:**
1.  **Think Strategically:** Do not think about small developer tasks. Your job is to define the core, high-level functional blocks of the project.
2.  **Strict Quantity Control:** You MUST break the project into exactly 3 to 5 core Epics. Do NOT create more than 5.
3.  **Output Format:** Your response MUST be a raw JSON object, starting with `{` and ending with `}`. It must contain a single root key "tasks", which is a list of task objects.
4.  **Task Object Schema:** Each task object must have: `id` (string), `name` (string, the Epic's title), `description` (string, what this Epic is about and its goal), and `is_complex` (boolean, which must always be `true`).

---
**EXAMPLE**

*Input:*
```
# Project Requirements: Simple Todo App

## 1. Vision & Goals
Create a basic todo application for personal task management.

## 2. Core Features
- Task creation and editing
- Task completion tracking
- Simple categorization
- Basic search functionality

## 3. Target Users
Individual users who need simple task management.

## 4. Technical Constraints
- Web-based application
- Simple and intuitive interface
```

*Output:*
```json
{
  "tasks": [
    {
      "id": "epic-001",
      "name": "User Interface Design & Implementation",
      "description": "Design and implement the core user interface for task management including task creation, editing, and display components.",
      "is_complex": true
    },
    {
      "id": "epic-002", 
      "name": "Task Management Core Logic",
      "description": "Implement the core business logic for task operations including CRUD operations, status management, and data persistence.",
      "is_complex": true
    },
    {
      "id": "epic-003",
      "name": "Search and Categorization Features",
      "description": "Build search functionality and categorization system to help users organize and find their tasks efficiently.",
      "is_complex": true
    }
  ]
}
```
---
"""

    def __init__(self, api_key: Optional[str] = None, base_url: Optional[str] = None):
        self.api_key = api_key or os.environ.get("OPENAI_API_KEY")
        self.base_url = base_url or os.environ.get("OPENAI_BASE_URL", "https://api.openai.com/v1")
        if not self.api_key:
            raise ValueError("API key must be provided or set as environment variable (OPENAI_API_KEY).")

    def parse_document_to_tasks(self, document: str) -> List[Dict]:
        """
        Parse a requirements document into a structured list of high-level tasks.
        """
        print("\n--- 任务生成Agent (使用 requests) ---")
        print("⏳ 正在调用LLM，将需求解析为JSON格式的任务列表...")
        
        api_url = f"{self.base_url}/chat/completions"
        if not self.base_url.endswith('/v1'):
            api_url = f"{self.base_url}/v1/chat/completions"
        
        headers = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.api_key}"
        }
        
        payload = {
            "model": "gpt-4o",
            "messages": [
                {"role": "system", "content": self.SYSTEM_PROMPT_TEMPLATE},
                {"role": "user", "content": document}
            ],
            "temperature": 0.05,
        }

        try:
            response = requests.post(api_url, headers=headers, json=payload, timeout=120)
            response.raise_for_status()
            
            response_data = response.json()
            content = response_data.get("choices", [{}])[0].get("message", {}).get("content", "")
            
            if not content:
                print("❌ LLM返回了空内容。")
                return []
            
            # Try to parse JSON directly
            try:
                result = json.loads(content)
                tasks = result.get("tasks", [])
                print(f"✅ 成功解析出 {len(tasks)} 个高级任务。")
                return tasks
            except json.JSONDecodeError:
                # Try to extract JSON from the content
                import re
                json_match = re.search(r'\{.*\}', content, re.DOTALL)
                if json_match:
                    try:
                        result = json.loads(json_match.group())
                        tasks = result.get("tasks", [])
                        print(f"✅ 从响应中提取并解析出 {len(tasks)} 个高级任务。")
                        return tasks
                    except json.JSONDecodeError:
                        pass
                
                print(f"❌ 无法解析LLM响应为JSON: {content}")
                return []
                
        except requests.exceptions.RequestException as e:
            print(f"❌ API调用失败: {e}")
            return []
        except Exception as e:
            print(f"❌ 未知错误: {e}")
            return []
