"""
Task Assignment MCP Server

A Model Context Protocol server that provides intelligent task assignment capabilities.
This server implements task decomposition, capability matching, smart assignment,
team collaboration, and daily review functionalities.
"""

import os
import asyncio
import logging
from typing import Dict, List, Any, Optional, Union
from datetime import datetime, timedelta
from contextlib import asynccontextmanager
from collections.abc import AsyncIterator

from mcp.server.fastmcp import FastMCP, Context
from pydantic import BaseModel, Field

# Import shared components
import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from shared.models import Task, HumanResource, AgentResource, Resource, TaskStatus, TaskPriority
from shared.agents.task_decomposition_agent import TaskDecompositionAgent
from shared.agents.dependency_manager import DependencyManager

# Configure logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


class TaskDecompositionResult(BaseModel):
    """Result of task decomposition"""
    parent_task_id: str = Field(description="ID of the parent task")
    subtasks: List[Task] = Field(description="Generated subtasks")
    decomposition_reasoning: str = Field(description="Explanation of decomposition approach")
    estimated_total_time: float = Field(description="Total estimated time for all subtasks")


class AssignmentRecommendation(BaseModel):
    """Task assignment recommendation"""
    task_id: str = Field(description="Task to be assigned")
    recommended_resource_id: str = Field(description="Recommended resource ID")
    resource_type: str = Field(description="Type of resource (human/agent)")
    confidence_score: float = Field(description="Confidence in the recommendation (0-1)")
    reasoning: str = Field(description="Explanation for the recommendation")
    estimated_completion_time: float = Field(description="Estimated completion time in hours")
    skill_match_score: float = Field(description="How well skills match (0-1)")
    availability_score: float = Field(description="Resource availability score (0-1)")


class TeamCollaborationPlan(BaseModel):
    """Team collaboration plan for complex tasks"""
    task_id: str = Field(description="Main task ID")
    collaboration_type: str = Field(description="Type of collaboration needed")
    team_members: List[str] = Field(description="List of resource IDs involved")
    coordination_strategy: str = Field(description="How team members should coordinate")
    communication_plan: str = Field(description="Communication requirements")
    milestone_schedule: List[Dict[str, Any]] = Field(description="Key milestones and deadlines")


class DailyReportReview(BaseModel):
    """Daily report review and scoring"""
    report_date: datetime = Field(description="Date of the report")
    team_member_id: str = Field(description="ID of the team member")
    productivity_score: float = Field(description="Productivity score (0-10)")
    quality_score: float = Field(description="Quality score (0-10)")
    collaboration_score: float = Field(description="Collaboration score (0-10)")
    overall_score: float = Field(description="Overall performance score (0-10)")
    feedback: str = Field(description="Detailed feedback and suggestions")
    improvement_areas: List[str] = Field(description="Areas for improvement")
    achievements: List[str] = Field(description="Notable achievements")


class AppContext(BaseModel):
    """Application context with dependencies"""
    task_decomposition_agent: Optional[TaskDecompositionAgent] = None
    dependency_manager: Optional[DependencyManager] = None
    tasks: Dict[str, Task] = Field(default_factory=dict)
    resources: Dict[str, Resource] = Field(default_factory=dict)
    assignments: Dict[str, str] = Field(default_factory=dict)  # task_id -> resource_id
    collaboration_plans: Dict[str, TeamCollaborationPlan] = Field(default_factory=dict)
    daily_reports: List[DailyReportReview] = Field(default_factory=list)


@asynccontextmanager
async def app_lifespan(server: FastMCP) -> AsyncIterator[AppContext]:
    """Manage application lifecycle with dependencies"""
    logger.info("Initializing Task Assignment MCP Server...")
    
    # Initialize AI agents
    api_key = os.environ.get("OPENAI_API_KEY")
    base_url = os.environ.get("OPENAI_BASE_URL", "https://api.openai.com/v1")
    
    if not api_key:
        logger.warning("OPENAI_API_KEY not set. AI features will be limited.")
    
    task_decomposition_agent = TaskDecompositionAgent(api_key=api_key, base_url=base_url) if api_key else None
    dependency_manager = DependencyManager(tasks={}, api_key=api_key, base_url=base_url) if api_key else None
    
    # Initialize with sample resources
    sample_resources = {
        "human_001": HumanResource(
            id="human_001",
            name="Alice Johnson",
            skills=["project_management", "requirements_analysis", "documentation"],
            load=0.3,
            cost_per_hour=85.0,
            status="available"
        ),
        "human_002": HumanResource(
            id="human_002", 
            name="Bob Smith",
            skills=["python", "backend_development", "api_design", "database"],
            load=0.7,
            cost_per_hour=95.0,
            status="available"
        ),
        "agent_001": AgentResource(
            id="agent_001",
            name="Code Generation Agent",
            skills=["code_generation", "unit_testing", "documentation"],
            load=0.2,
            cost_per_task=0.5,
            system_prompt="You are a code generation specialist. Generate clean, well-documented code.",
            status="available"
        ),
        "agent_002": AgentResource(
            id="agent_002",
            name="QA Testing Agent", 
            skills=["testing", "quality_assurance", "bug_detection"],
            load=0.1,
            cost_per_task=0.3,
            system_prompt="You are a QA specialist. Focus on thorough testing and quality assurance.",
            status="available"
        )
    }
    
    context = AppContext(
        task_decomposition_agent=task_decomposition_agent,
        dependency_manager=dependency_manager,
        resources=sample_resources
    )
    
    logger.info("Task Assignment MCP Server initialized successfully")
    
    try:
        yield context
    finally:
        logger.info("Shutting down Task Assignment MCP Server...")


# Create the MCP server
mcp = FastMCP("Task Assignment MCP", lifespan=app_lifespan)


@mcp.tool()
def decompose_complex_task(
    task_id: str,
    ctx: Context,
    max_subtasks: int = 5,
    target_time_per_subtask: float = 4.0
) -> TaskDecompositionResult:
    """
    Decompose a complex task into manageable subtasks.
    
    Uses AI to intelligently break down large tasks into smaller,
    more manageable pieces with clear deliverables.
    """
    app_ctx = ctx.request_context.lifespan_context
    
    task = app_ctx.tasks.get(task_id)
    if not task:
        raise ValueError(f"Task {task_id} not found")
    
    if not app_ctx.task_decomposition_agent:
        # Fallback decomposition without AI
        subtasks = []
        if task.time_cost > target_time_per_subtask:
            num_subtasks = min(max_subtasks, int(task.time_cost / target_time_per_subtask))
            time_per_subtask = task.time_cost / num_subtasks
            
            for i in range(num_subtasks):
                subtask = Task(
                    id=f"{task_id}_sub_{i+1}",
                    name=f"{task.name} - Part {i+1}",
                    description=f"Subtask {i+1} of {task.name}",
                    time_cost=time_per_subtask,
                    priority=task.priority,
                    project_id=task.project_id
                )
                subtasks.append(subtask)
                app_ctx.tasks[subtask.id] = subtask
        
        return TaskDecompositionResult(
            parent_task_id=task_id,
            subtasks=subtasks,
            decomposition_reasoning="Simple time-based decomposition (AI not available)",
            estimated_total_time=sum(st.time_cost for st in subtasks)
        )
    
    try:
        # Use AI for intelligent decomposition
        existing_task_ids = list(app_ctx.tasks.keys())
        raw_subtasks = app_ctx.task_decomposition_agent.decompose_task(
            task.dict(), existing_task_ids
        )
        
        subtasks = []
        for raw_subtask in raw_subtasks:
            subtask = Task(
                id=raw_subtask.get("id", f"{task_id}_sub_{len(subtasks)+1}"),
                name=raw_subtask.get("name", f"Subtask {len(subtasks)+1}"),
                description=raw_subtask.get("description", ""),
                time_cost=float(raw_subtask.get("time_cost", target_time_per_subtask)),
                priority=task.priority,
                project_id=task.project_id,
                dependencies=raw_subtask.get("dependencies", [])
            )
            subtasks.append(subtask)
            app_ctx.tasks[subtask.id] = subtask
        
        # Update parent task to reference subtasks
        task.subtasks = [st.id for st in subtasks]
        
        return TaskDecompositionResult(
            parent_task_id=task_id,
            subtasks=subtasks,
            decomposition_reasoning="AI-powered intelligent decomposition based on task complexity and requirements",
            estimated_total_time=sum(st.time_cost for st in subtasks)
        )
        
    except Exception as e:
        logger.error(f"Error in task decomposition: {e}")
        # Fallback to simple decomposition
        return decompose_complex_task(task_id, ctx, max_subtasks, target_time_per_subtask)


@mcp.tool()
def recommend_task_assignment(
    task_id: str,
    ctx: Context,
    consider_workload: bool = True,
    prefer_human: bool = False
) -> AssignmentRecommendation:
    """
    Recommend the best resource to assign a task to.

    Analyzes task requirements, resource capabilities, current workload,
    and availability to make optimal assignment recommendations.
    """
    app_ctx = ctx.request_context.lifespan_context

    task = app_ctx.tasks.get(task_id)
    if not task:
        raise ValueError(f"Task {task_id} not found")

    best_resource = None
    best_score = 0.0
    best_reasoning = ""

    for resource_id, resource in app_ctx.resources.items():
        score = 0.0
        reasoning_parts = []

        # Skill matching score
        task_keywords = (task.name + " " + task.description).lower()
        skill_matches = sum(1 for skill in resource.skills if skill.lower() in task_keywords)
        skill_score = min(skill_matches * 25, 100)  # Max 100 points for skills
        score += skill_score
        reasoning_parts.append(f"Skill match: {skill_score}")

        # Availability score
        if consider_workload:
            if isinstance(resource, HumanResource):
                availability = max(0, 1.0 - resource.load)
                availability_score = availability * 50
            else:  # AgentResource
                availability = max(0, 1.0 - resource.load / 5.0)  # Agents can handle more parallel tasks
                availability_score = availability * 50
        else:
            availability_score = 50

        score += availability_score
        reasoning_parts.append(f"Availability: {availability_score}")

        # Resource type preference
        if prefer_human and isinstance(resource, HumanResource):
            type_bonus = 20
        elif not prefer_human and isinstance(resource, AgentResource):
            type_bonus = 15
        else:
            type_bonus = 0

        score += type_bonus
        if type_bonus > 0:
            reasoning_parts.append(f"Type preference: {type_bonus}")

        # Cost efficiency (lower cost = higher score)
        if isinstance(resource, HumanResource):
            cost_score = max(0, 100 - resource.cost_per_hour)
        else:
            cost_score = max(0, 100 - resource.cost_per_task * 10)

        score += cost_score * 0.3  # Weight cost less than skills/availability
        reasoning_parts.append(f"Cost efficiency: {cost_score * 0.3:.1f}")

        if score > best_score:
            best_score = score
            best_resource = resource
            best_reasoning = "; ".join(reasoning_parts)

    if not best_resource:
        return AssignmentRecommendation(
            task_id=task_id,
            recommended_resource_id="",
            resource_type="none",
            confidence_score=0.0,
            reasoning="No suitable resources found",
            estimated_completion_time=task.time_cost,
            skill_match_score=0.0,
            availability_score=0.0
        )

    # Calculate estimated completion time based on resource efficiency
    if isinstance(best_resource, HumanResource):
        efficiency_factor = 1.0 + (len(best_resource.skills) * 0.1)  # More skills = faster
        completion_time = task.time_cost / efficiency_factor
    else:
        completion_time = task.time_cost * 0.8  # Agents are generally faster for routine tasks

    return AssignmentRecommendation(
        task_id=task_id,
        recommended_resource_id=best_resource.id,
        resource_type=best_resource.type,
        confidence_score=min(best_score / 200, 1.0),  # Normalize to 0-1
        reasoning=f"Best match: {best_resource.name} - {best_reasoning}",
        estimated_completion_time=completion_time,
        skill_match_score=skill_score / 100,
        availability_score=availability_score / 50
    )


@mcp.tool()
def assign_task_to_resource(
    task_id: str,
    resource_id: str,
    ctx: Context,
    start_date: Optional[str] = None,
    notes: str = ""
) -> Dict[str, Any]:
    """
    Assign a task to a specific resource.

    Creates the assignment, updates resource workload, and sets up
    tracking for the task execution.
    """
    app_ctx = ctx.request_context.lifespan_context

    task = app_ctx.tasks.get(task_id)
    if not task:
        raise ValueError(f"Task {task_id} not found")

    resource = app_ctx.resources.get(resource_id)
    if not resource:
        raise ValueError(f"Resource {resource_id} not found")

    # Update task assignment
    task.assignee = resource_id
    task.status = TaskStatus.TODO
    if start_date:
        task.estimated_completion = datetime.fromisoformat(start_date) + timedelta(hours=task.time_cost)

    # Update resource workload
    if isinstance(resource, HumanResource):
        resource.load = min(1.0, resource.load + (task.time_cost / 40))  # Assume 40-hour work week
    else:
        resource.load += 1  # Simple task count for agents

    # Record assignment
    app_ctx.assignments[task_id] = resource_id

    return {
        "task_id": task_id,
        "resource_id": resource_id,
        "resource_name": resource.name,
        "assignment_date": datetime.now().isoformat(),
        "estimated_completion": task.estimated_completion.isoformat() if task.estimated_completion else None,
        "notes": notes,
        "status": "assigned"
    }


@mcp.tool()
def create_collaboration_plan(
    task_id: str,
    ctx: Context,
    required_skills: Optional[List[str]] = None,
    max_team_size: int = 4
) -> TeamCollaborationPlan:
    """
    Create a collaboration plan for complex tasks requiring multiple resources.

    Identifies the best team composition and coordination strategy for
    tasks that require diverse skills or parallel execution.
    """
    app_ctx = ctx.request_context.lifespan_context

    task = app_ctx.tasks.get(task_id)
    if not task:
        raise ValueError(f"Task {task_id} not found")

    # Determine required skills from task or use provided ones
    if not required_skills:
        task_text = (task.name + " " + task.description).lower()
        required_skills = []
        for resource in app_ctx.resources.values():
            for skill in resource.skills:
                if skill.lower() in task_text and skill not in required_skills:
                    required_skills.append(skill)

    # Select team members based on skills and availability
    team_members = []
    covered_skills = set()

    # Sort resources by availability and skill relevance
    available_resources = [
        (rid, r) for rid, r in app_ctx.resources.items()
        if r.status == "available" and r.load < 0.8
    ]

    for resource_id, resource in available_resources:
        if len(team_members) >= max_team_size:
            break

        # Check if this resource adds valuable skills
        resource_skills = set(resource.skills)
        new_skills = resource_skills - covered_skills

        if new_skills or len(team_members) == 0:  # Always include at least one member
            team_members.append(resource_id)
            covered_skills.update(resource_skills)

    # Determine collaboration type
    if len(team_members) <= 2:
        collaboration_type = "pair_programming"
        coordination_strategy = "Direct collaboration with shared responsibility"
    elif task.time_cost > 20:
        collaboration_type = "parallel_development"
        coordination_strategy = "Divide task into parallel workstreams with regular sync points"
    else:
        collaboration_type = "sequential_handoff"
        coordination_strategy = "Sequential execution with clear handoff points"

    # Create communication plan
    if len(team_members) <= 2:
        communication_plan = "Direct communication as needed, daily check-ins"
    else:
        communication_plan = "Daily standup meetings, shared documentation, milestone reviews"

    # Generate milestone schedule
    milestones = []
    if task.time_cost > 8:
        milestone_count = min(4, int(task.time_cost / 4))
        for i in range(milestone_count):
            milestone_date = datetime.now() + timedelta(days=(i + 1) * 2)
            milestones.append({
                "milestone": f"Milestone {i + 1}",
                "date": milestone_date.isoformat(),
                "deliverable": f"Progress checkpoint {i + 1}",
                "responsible": team_members[i % len(team_members)] if team_members else ""
            })

    collaboration_plan = TeamCollaborationPlan(
        task_id=task_id,
        collaboration_type=collaboration_type,
        team_members=team_members,
        coordination_strategy=coordination_strategy,
        communication_plan=communication_plan,
        milestone_schedule=milestones
    )

    app_ctx.collaboration_plans[task_id] = collaboration_plan
    return collaboration_plan


@mcp.tool()
def review_daily_report(
    team_member_id: str,
    report_content: str,
    ctx: Context,
    report_date: Optional[str] = None
) -> DailyReportReview:
    """
    Review and score a team member's daily report.

    Analyzes daily reports to provide feedback, scoring, and suggestions
    for improvement. Tracks performance trends over time.
    """
    app_ctx = ctx.request_context.lifespan_context

    resource = app_ctx.resources.get(team_member_id)
    if not resource:
        raise ValueError(f"Team member {team_member_id} not found")

    report_date_obj = datetime.fromisoformat(report_date) if report_date else datetime.now()

    # Analyze report content (simplified scoring algorithm)
    content_lower = report_content.lower()

    # Productivity scoring based on keywords and content length
    productivity_keywords = ["completed", "finished", "delivered", "implemented", "resolved", "achieved"]
    productivity_score = min(10, len([kw for kw in productivity_keywords if kw in content_lower]) * 2 + len(report_content) / 100)

    # Quality scoring based on detail and specific mentions
    quality_keywords = ["tested", "reviewed", "documented", "validated", "optimized", "refactored"]
    quality_score = min(10, len([kw for kw in quality_keywords if kw in content_lower]) * 2.5 + (3 if len(report_content) > 200 else 1))

    # Collaboration scoring based on team interaction mentions
    collaboration_keywords = ["discussed", "collaborated", "helped", "reviewed", "paired", "meeting", "feedback"]
    collaboration_score = min(10, len([kw for kw in collaboration_keywords if kw in content_lower]) * 3 + 2)

    # Overall score (weighted average)
    overall_score = (productivity_score * 0.4 + quality_score * 0.4 + collaboration_score * 0.2)

    # Generate feedback based on scores
    feedback_parts = []
    achievements = []
    improvement_areas = []

    if productivity_score >= 8:
        achievements.append("High productivity with multiple completed tasks")
    elif productivity_score < 5:
        improvement_areas.append("Focus on completing more tasks and deliverables")
        feedback_parts.append("Consider breaking down large tasks into smaller, manageable pieces")

    if quality_score >= 8:
        achievements.append("Strong attention to quality and detail")
    elif quality_score < 5:
        improvement_areas.append("Increase focus on testing and documentation")
        feedback_parts.append("Ensure all work is properly tested and documented")

    if collaboration_score >= 7:
        achievements.append("Good team collaboration and communication")
    elif collaboration_score < 4:
        improvement_areas.append("Increase team interaction and collaboration")
        feedback_parts.append("Participate more actively in team discussions and code reviews")

    # Add general feedback
    if overall_score >= 8:
        feedback_parts.append("Excellent work overall! Keep up the high standards.")
    elif overall_score >= 6:
        feedback_parts.append("Good performance with room for improvement in identified areas.")
    else:
        feedback_parts.append("Performance needs improvement. Focus on the suggested areas.")

    review = DailyReportReview(
        report_date=report_date_obj,
        team_member_id=team_member_id,
        productivity_score=productivity_score,
        quality_score=quality_score,
        collaboration_score=collaboration_score,
        overall_score=overall_score,
        feedback=". ".join(feedback_parts),
        improvement_areas=improvement_areas,
        achievements=achievements
    )

    app_ctx.daily_reports.append(review)
    return review


# Resources
@mcp.resource("resource://{resource_id}")
def get_resource_details(resource_id: str, ctx: Context) -> str:
    """Get detailed information about a specific resource (human or agent)"""
    app_ctx = ctx.request_context.lifespan_context

    resource = app_ctx.resources.get(resource_id)
    if not resource:
        return f"Resource {resource_id} not found"

    if isinstance(resource, HumanResource):
        return f"""
# Human Resource: {resource.name}

**ID:** {resource.id}
**Type:** Human
**Status:** {resource.status}
**Current Load:** {resource.load * 100:.1f}%

## Skills
{', '.join(resource.skills) if resource.skills else 'No specific skills listed'}

## Cost & Availability
- **Cost per Hour:** ${resource.cost_per_hour:.2f}
- **Availability:** {(1 - resource.load) * 100:.1f}%

## Current Assignments
{_get_resource_assignments(resource_id, ctx)}
"""
    else:  # AgentResource
        return f"""
# AI Agent: {resource.name}

**ID:** {resource.id}
**Type:** AI Agent
**Status:** {resource.status}
**Current Load:** {resource.load} active tasks

## Capabilities
{', '.join(resource.skills) if resource.skills else 'No specific capabilities listed'}

## Configuration
- **Cost per Task:** ${resource.cost_per_task:.2f}
- **System Prompt:** {resource.system_prompt[:100]}...

## Available Tools
{', '.join([tool.get('name', 'Unknown') for tool in resource.tools]) if resource.tools else 'No specific tools'}

## Current Assignments
{_get_resource_assignments(resource_id, ctx)}
"""


def _get_resource_assignments(resource_id: str, ctx: Context) -> str:
    """Helper function to get current assignments for a resource"""
    app_ctx = ctx.request_context.lifespan_context

    assigned_tasks = [
        task for task in app_ctx.tasks.values()
        if task.assignee == resource_id and task.status != TaskStatus.COMPLETED
    ]

    if not assigned_tasks:
        return "No current assignments"

    assignment_list = []
    for task in assigned_tasks:
        status_emoji = {"todo": "📋", "in_progress": "🔄", "blocked": "🚫"}.get(task.status.value, "❓")
        assignment_list.append(f"- {status_emoji} **{task.name}** ({task.time_cost}h)")

    return '\n'.join(assignment_list)


@mcp.resource("team://{project_id}/workload")
def get_team_workload_summary(project_id: str, ctx: Context) -> str:
    """Get team workload summary for a project"""
    app_ctx = ctx.request_context.lifespan_context

    # Get project tasks
    project_tasks = [t for t in app_ctx.tasks.values() if t.project_id == project_id]

    if not project_tasks:
        return f"No tasks found for project {project_id}"

    # Analyze workload distribution
    resource_workload = {}
    unassigned_tasks = []

    for task in project_tasks:
        if task.assignee:
            if task.assignee not in resource_workload:
                resource_workload[task.assignee] = {"tasks": 0, "hours": 0.0, "completed": 0}
            resource_workload[task.assignee]["tasks"] += 1
            resource_workload[task.assignee]["hours"] += task.time_cost
            if task.status == TaskStatus.COMPLETED:
                resource_workload[task.assignee]["completed"] += 1
        else:
            unassigned_tasks.append(task)

    # Generate summary
    summary_parts = [f"# Team Workload Summary - Project {project_id}\n"]

    if resource_workload:
        summary_parts.append("## Resource Allocation")
        for resource_id, workload in resource_workload.items():
            resource = app_ctx.resources.get(resource_id)
            resource_name = resource.name if resource else resource_id
            completion_rate = (workload["completed"] / workload["tasks"] * 100) if workload["tasks"] > 0 else 0

            summary_parts.append(f"""
### {resource_name} ({resource_id})
- **Total Tasks:** {workload["tasks"]}
- **Total Hours:** {workload["hours"]:.1f}
- **Completed:** {workload["completed"]} ({completion_rate:.1f}%)
- **Current Load:** {app_ctx.resources.get(resource_id, {}).get('load', 'Unknown') if resource_id in app_ctx.resources else 'Unknown'}
""")

    if unassigned_tasks:
        summary_parts.append(f"\n## Unassigned Tasks ({len(unassigned_tasks)})")
        for task in unassigned_tasks[:5]:  # Show first 5 unassigned tasks
            summary_parts.append(f"- **{task.name}** ({task.time_cost}h) - Priority: {TaskPriority(task.priority).name}")

        if len(unassigned_tasks) > 5:
            summary_parts.append(f"... and {len(unassigned_tasks) - 5} more")

    return '\n'.join(summary_parts)


# Prompts
@mcp.prompt()
def assignment_optimization_prompt(project_id: str, ctx: Context) -> str:
    """Generate a prompt for optimizing task assignments across a project"""
    app_ctx = ctx.request_context.lifespan_context

    # Get project tasks and assignments
    project_tasks = [t for t in app_ctx.tasks.values() if t.project_id == project_id]

    if not project_tasks:
        return f"No tasks found for project {project_id}"

    # Analyze current assignments
    assigned_tasks = [t for t in project_tasks if t.assignee]
    unassigned_tasks = [t for t in project_tasks if not t.assignee]

    # Get resource utilization
    resource_workload = {}
    for task in assigned_tasks:
        if task.assignee not in resource_workload:
            resource_workload[task.assignee] = {"tasks": 0, "hours": 0.0}
        resource_workload[task.assignee]["tasks"] += 1
        resource_workload[task.assignee]["hours"] += task.time_cost

    return f"""Analyze and optimize the task assignments for project {project_id}:

**Current Assignment Status:**
- Total tasks: {len(project_tasks)}
- Assigned tasks: {len(assigned_tasks)}
- Unassigned tasks: {len(unassigned_tasks)}

**Resource Utilization:**
{chr(10).join(f'- {rid}: {data["tasks"]} tasks, {data["hours"]} hours' for rid, data in resource_workload.items())}

**Unassigned Tasks:**
{chr(10).join(f'- {t.name} ({t.time_cost}h, Priority: {TaskPriority(t.priority).name})' for t in unassigned_tasks[:10])}

Please provide:
1. **Workload Balance Analysis:** Are resources evenly distributed? Who is over/under-utilized?
2. **Skill Matching Assessment:** Are tasks assigned to resources with appropriate skills?
3. **Priority Optimization:** Are high-priority tasks assigned to the most capable resources?
4. **Bottleneck Identification:** Which resources or dependencies might cause delays?
5. **Reassignment Recommendations:** Specific suggestions for improving assignments
6. **Timeline Impact:** How would optimizations affect project completion time?

Focus on actionable recommendations that can be implemented immediately."""


@mcp.prompt()
def team_performance_analysis_prompt(team_member_ids: List[str], ctx: Context) -> str:
    """Generate a prompt for analyzing team performance trends"""
    app_ctx = ctx.request_context.lifespan_context

    if not team_member_ids:
        return "No team members specified for analysis"

    # Get recent reports for team members
    recent_reports = [r for r in app_ctx.daily_reports if r.team_member_id in team_member_ids]

    if not recent_reports:
        return "No daily reports found for the specified team members"

    # Organize reports by team member
    member_reports = {}
    for report in recent_reports:
        if report.team_member_id not in member_reports:
            member_reports[report.team_member_id] = []
        member_reports[report.team_member_id].append(report)

    # Calculate averages
    team_stats = {}
    for member_id, reports in member_reports.items():
        if reports:
            team_stats[member_id] = {
                "report_count": len(reports),
                "avg_productivity": sum(r.productivity_score for r in reports) / len(reports),
                "avg_quality": sum(r.quality_score for r in reports) / len(reports),
                "avg_collaboration": sum(r.collaboration_score for r in reports) / len(reports),
                "avg_overall": sum(r.overall_score for r in reports) / len(reports),
                "latest_report": max(reports, key=lambda r: r.report_date)
            }

    return f"""Analyze team performance trends for {len(team_member_ids)} team members:

**Team Performance Summary:**
{chr(10).join(f'''- **{member_id}**: {stats["report_count"]} reports
  - Productivity: {stats["avg_productivity"]:.1f}/10
  - Quality: {stats["avg_quality"]:.1f}/10
  - Collaboration: {stats["avg_collaboration"]:.1f}/10
  - Overall: {stats["avg_overall"]:.1f}/10''' for member_id, stats in team_stats.items())}

**Recent Achievements:**
{chr(10).join(f'- {member_id}: {", ".join(stats["latest_report"].achievements[:3])}' for member_id, stats in team_stats.items() if stats["latest_report"].achievements)}

**Improvement Areas:**
{chr(10).join(f'- {member_id}: {", ".join(stats["latest_report"].improvement_areas[:2])}' for member_id, stats in team_stats.items() if stats["latest_report"].improvement_areas)}

Please provide:
1. **Performance Trends:** Are team members improving, declining, or stable?
2. **Strengths Analysis:** What are each member's key strengths?
3. **Development Opportunities:** Where can each member improve?
4. **Team Dynamics:** How well is the team collaborating?
5. **Resource Allocation:** Should task assignments be adjusted based on performance?
6. **Training Recommendations:** What skills development would benefit the team?
7. **Recognition Opportunities:** Who deserves recognition for outstanding work?

Focus on constructive feedback and actionable development plans."""


def create_server() -> FastMCP:
    """Create and return the configured MCP server"""
    return mcp


if __name__ == "__main__":
    # Run the server
    mcp.run()
