"""
Task Management MCP Server

A Model Context Protocol server that provides intelligent task management capabilities.
This server implements task analysis, priority scheduling, risk prediction, reminder systems,
progress tracking, and task review functionalities.
"""

import os
import asyncio
import logging
from typing import Dict, List, Any, Optional
from datetime import datetime, timedelta
from contextlib import asynccontextmanager
from collections.abc import AsyncIterator

from mcp.server.fastmcp import FastMCP, Context
from pydantic import BaseModel, Field

# Import shared components
import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from shared.models import Task, Project, TaskStatus, TaskPriority, RiskAssessment, ProgressUpdate, TaskRecommendation, Reminder
from shared.agents.requirement_agent import RequirementAgent
from shared.agents.task_generation_agent import TaskGenerationAgent

# Configure logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


class TaskAnalysisResult(BaseModel):
    """Result of task analysis"""
    tasks: List[Task] = Field(description="Analyzed and structured tasks")
    total_estimated_time: float = Field(description="Total estimated time in hours")
    priority_distribution: Dict[str, int] = Field(description="Distribution of tasks by priority")
    risk_summary: str = Field(description="Overall risk assessment summary")


class ScheduleRecommendation(BaseModel):
    """Schedule recommendation result"""
    recommended_order: List[str] = Field(description="Recommended task execution order (task IDs)")
    estimated_completion_date: datetime = Field(description="Estimated project completion date")
    critical_path: List[str] = Field(description="Critical path task IDs")
    parallel_opportunities: List[List[str]] = Field(description="Tasks that can be executed in parallel")


class AppContext(BaseModel):
    """Application context with dependencies"""
    requirement_agent: RequirementAgent
    task_generation_agent: TaskGenerationAgent
    tasks: Dict[str, Task] = Field(default_factory=dict)
    projects: Dict[str, Project] = Field(default_factory=dict)
    reminders: Dict[str, Reminder] = Field(default_factory=dict)


@asynccontextmanager
async def app_lifespan(server: FastMCP) -> AsyncIterator[AppContext]:
    """Manage application lifecycle with dependencies"""
    logger.info("Initializing Task Management MCP Server...")
    
    # Initialize AI agents
    api_key = os.environ.get("OPENAI_API_KEY")
    base_url = os.environ.get("OPENAI_BASE_URL", "https://api.openai.com/v1")
    
    if not api_key:
        logger.warning("OPENAI_API_KEY not set. AI features will be limited.")
        
    requirement_agent = RequirementAgent(api_key=api_key, base_url=base_url) if api_key else None
    task_generation_agent = TaskGenerationAgent(api_key=api_key, base_url=base_url) if api_key else None
    
    context = AppContext(
        requirement_agent=requirement_agent,
        task_generation_agent=task_generation_agent
    )
    
    logger.info("Task Management MCP Server initialized successfully")
    
    try:
        yield context
    finally:
        logger.info("Shutting down Task Management MCP Server...")


# Create the MCP server
mcp = FastMCP("Task Management MCP", lifespan=app_lifespan)


@mcp.tool()
def analyze_task_input(
    task_description: str,
    ctx: Context,
    project_id: Optional[str] = None,
    due_date: Optional[str] = None,
    priority_hint: Optional[str] = None
) -> TaskAnalysisResult:
    """
    Analyze natural language task input and extract structured information.
    
    This tool uses AI to identify tasks, subtasks, deadlines, dependencies,
    time costs, benefits, importance, and marginal costs from natural language input.
    """
    app_ctx = ctx.request_context.lifespan_context
    
    if not app_ctx.task_generation_agent:
        # Fallback to manual parsing when AI is not available
        task = Task(
            id=f"task_{len(app_ctx.tasks) + 1}",
            name=task_description[:100],  # Truncate long descriptions
            description=task_description,
            due_date=datetime.fromisoformat(due_date) if due_date else None,
            priority=TaskPriority.MEDIUM,
            project_id=project_id
        )
        app_ctx.tasks[task.id] = task
        
        return TaskAnalysisResult(
            tasks=[task],
            total_estimated_time=task.time_cost,
            priority_distribution={"medium": 1},
            risk_summary="Manual analysis - limited risk assessment available"
        )
    
    # Use AI to analyze the task input
    try:
        # Generate structured requirements first
        requirements = app_ctx.requirement_agent.generate_requirements_document(task_description)
        
        # Parse requirements into tasks
        raw_tasks = app_ctx.task_generation_agent.parse_document_to_tasks(requirements)
        
        # Convert to our Task model
        analyzed_tasks = []
        priority_dist = {"critical": 0, "high": 0, "medium": 0, "low": 0, "minimal": 0}
        total_time = 0.0
        
        for raw_task in raw_tasks:
            task = Task(
                id=f"task_{len(app_ctx.tasks) + len(analyzed_tasks) + 1}",
                name=raw_task.get("name", "Unnamed Task"),
                description=raw_task.get("description", ""),
                priority=TaskPriority(raw_task.get("priority", 3)),
                time_cost=float(raw_task.get("time_cost", 1.0)),
                importance=raw_task.get("importance", 3),
                benefit=raw_task.get("benefit", 5),
                marginal_cost=raw_task.get("marginal_cost", 3),
                due_date=datetime.fromisoformat(due_date) if due_date else None,
                project_id=project_id
            )
            
            analyzed_tasks.append(task)
            app_ctx.tasks[task.id] = task
            
            priority_name = TaskPriority(task.priority).name.lower()
            priority_dist[priority_name] += 1
            total_time += task.time_cost
        
        return TaskAnalysisResult(
            tasks=analyzed_tasks,
            total_estimated_time=total_time,
            priority_distribution=priority_dist,
            risk_summary=f"Analyzed {len(analyzed_tasks)} tasks with total estimated time of {total_time} hours"
        )
        
    except Exception as e:
        logger.error(f"Error analyzing task input: {e}")
        # Fallback to simple parsing
        task = Task(
            id=f"task_{len(app_ctx.tasks) + 1}",
            name=task_description[:100],
            description=task_description,
            project_id=project_id
        )
        app_ctx.tasks[task.id] = task
        
        return TaskAnalysisResult(
            tasks=[task],
            total_estimated_time=task.time_cost,
            priority_distribution={"medium": 1},
            risk_summary=f"Fallback analysis due to error: {str(e)}"
        )


@mcp.tool()
def get_schedule_recommendation(
    ctx: Context,
    project_id: Optional[str] = None,
    available_hours_per_day: float = 8.0,
    start_date: Optional[str] = None
) -> ScheduleRecommendation:
    """
    Generate intelligent scheduling recommendations for tasks.
    
    Provides optimal task ordering, estimated completion dates, critical path analysis,
    and identifies opportunities for parallel execution.
    """
    app_ctx = ctx.request_context.lifespan_context
    
    # Filter tasks by project if specified
    tasks = list(app_ctx.tasks.values())
    if project_id:
        tasks = [t for t in tasks if t.project_id == project_id]
    
    if not tasks:
        return ScheduleRecommendation(
            recommended_order=[],
            estimated_completion_date=datetime.now(),
            critical_path=[],
            parallel_opportunities=[]
        )
    
    # Sort tasks by priority and dependencies
    # This is a simplified scheduling algorithm
    sorted_tasks = sorted(tasks, key=lambda t: (t.priority, -t.importance, t.time_cost))
    
    # Calculate estimated completion date
    total_time = sum(t.time_cost for t in sorted_tasks)
    start = datetime.fromisoformat(start_date) if start_date else datetime.now()
    working_days = total_time / available_hours_per_day
    completion_date = start + timedelta(days=working_days)
    
    # Identify critical path (simplified - tasks with dependencies)
    critical_path = [t.id for t in sorted_tasks if t.dependencies]
    
    # Find parallel opportunities (tasks without dependencies)
    parallel_tasks = [t.id for t in sorted_tasks if not t.dependencies and t.id not in critical_path]
    parallel_opportunities = [parallel_tasks] if parallel_tasks else []
    
    return ScheduleRecommendation(
        recommended_order=[t.id for t in sorted_tasks],
        estimated_completion_date=completion_date,
        critical_path=critical_path,
        parallel_opportunities=parallel_opportunities
    )


@mcp.tool()
def predict_task_risks(
    task_id: str,
    ctx: Context,
    include_mitigation: bool = True
) -> RiskAssessment:
    """
    Predict potential risks for a specific task and suggest mitigation strategies.
    
    Analyzes task characteristics to identify potential risks and provides
    actionable mitigation strategies.
    """
    app_ctx = ctx.request_context.lifespan_context
    
    task = app_ctx.tasks.get(task_id)
    if not task:
        raise ValueError(f"Task {task_id} not found")
    
    # Risk assessment logic
    risk_factors = []
    risk_level = 1
    
    # Analyze various risk factors
    if task.time_cost > 8:
        risk_factors.append("Large time estimate increases uncertainty")
        risk_level += 1
    
    if task.dependencies:
        risk_factors.append("Dependencies on other tasks create scheduling risks")
        risk_level += 1
    
    if task.due_date and task.due_date < datetime.now() + timedelta(days=2):
        risk_factors.append("Tight deadline increases pressure and error risk")
        risk_level += 2
    
    if task.priority == TaskPriority.CRITICAL:
        risk_factors.append("Critical priority means high impact if delayed")
        risk_level += 1
    
    # Cap risk level at 5
    risk_level = min(risk_level, 5)
    
    # Generate mitigation strategies
    mitigation_strategies = []
    if include_mitigation:
        if "Large time estimate" in str(risk_factors):
            mitigation_strategies.append("Break down into smaller, more manageable subtasks")
        
        if "Dependencies" in str(risk_factors):
            mitigation_strategies.append("Identify alternative approaches that reduce dependencies")
        
        if "Tight deadline" in str(risk_factors):
            mitigation_strategies.append("Negotiate deadline extension or reduce scope")
        
        if "Critical priority" in str(risk_factors):
            mitigation_strategies.append("Assign most experienced team member and add buffer time")
    
    return RiskAssessment(
        task_id=task_id,
        risk_level=risk_level,
        risk_factors=risk_factors,
        mitigation_strategies=mitigation_strategies,
        probability=min(0.1 * risk_level + 0.2, 0.9),
        impact=task.importance
    )


@mcp.tool()
def create_reminder(
    task_id: str,
    reminder_type: str,
    ctx: Context,
    scheduled_time: Optional[str] = None,
    message: Optional[str] = None
) -> Reminder:
    """
    Create a reminder for a task (start, deadline, or progress check).

    Sets up automated reminders for task management including start notifications,
    deadline warnings, and progress check prompts.
    """
    app_ctx = ctx.request_context.lifespan_context

    task = app_ctx.tasks.get(task_id)
    if not task:
        raise ValueError(f"Task {task_id} not found")

    # Generate default scheduled time based on reminder type
    if not scheduled_time:
        if reminder_type == "start":
            scheduled_time = datetime.now().isoformat()
        elif reminder_type == "deadline" and task.due_date:
            # Remind 1 day before deadline
            scheduled_time = (task.due_date - timedelta(days=1)).isoformat()
        elif reminder_type == "progress":
            # Progress check in 2 days
            scheduled_time = (datetime.now() + timedelta(days=2)).isoformat()
        else:
            scheduled_time = datetime.now().isoformat()

    # Generate default message
    if not message:
        if reminder_type == "start":
            message = f"Time to start working on: {task.name}"
        elif reminder_type == "deadline":
            message = f"Deadline approaching for: {task.name}"
        elif reminder_type == "progress":
            message = f"Progress check needed for: {task.name}"
        else:
            message = f"Reminder for task: {task.name}"

    reminder = Reminder(
        id=f"reminder_{len(app_ctx.reminders) + 1}",
        task_id=task_id,
        reminder_type=reminder_type,
        message=message,
        scheduled_time=datetime.fromisoformat(scheduled_time)
    )

    app_ctx.reminders[reminder.id] = reminder
    return reminder


@mcp.tool()
def update_task_progress(
    task_id: str,
    progress: float,
    ctx: Context,
    time_spent: float = 0.0,
    notes: str = "",
    updated_by: str = "user"
) -> ProgressUpdate:
    """
    Update task progress and dynamically adjust estimates.

    Records progress updates and automatically adjusts remaining time estimates
    and task priorities based on actual progress.
    """
    app_ctx = ctx.request_context.lifespan_context

    task = app_ctx.tasks.get(task_id)
    if not task:
        raise ValueError(f"Task {task_id} not found")

    # Validate progress
    if not 0.0 <= progress <= 1.0:
        raise ValueError("Progress must be between 0.0 and 1.0")

    # Update task progress
    old_progress = task.progress
    task.progress = progress
    task.actual_time_spent += time_spent
    task.updated_at = datetime.now()

    # Adjust status based on progress
    if progress == 1.0:
        task.status = TaskStatus.COMPLETED
    elif progress > 0.0:
        task.status = TaskStatus.IN_PROGRESS

    # Create progress update record
    progress_update = ProgressUpdate(
        task_id=task_id,
        progress=progress,
        time_spent=time_spent,
        notes=notes,
        updated_by=updated_by
    )

    return progress_update


@mcp.tool()
def get_next_task_recommendation(
    ctx: Context,
    available_time: float = 2.0,
    current_context: str = "",
    exclude_task_ids: Optional[List[str]] = None
) -> TaskRecommendation:
    """
    Get intelligent recommendation for the next task to work on.

    Analyzes current situation, available time, and task priorities to recommend
    the most suitable next task.
    """
    app_ctx = ctx.request_context.lifespan_context
    exclude_task_ids = exclude_task_ids or []

    # Filter available tasks
    available_tasks = [
        task for task in app_ctx.tasks.values()
        if (task.status in [TaskStatus.TODO, TaskStatus.IN_PROGRESS] and
            task.id not in exclude_task_ids and
            task.time_cost <= available_time * 1.5)  # Allow some flexibility
    ]

    if not available_tasks:
        return TaskRecommendation(
            task_id="",
            score=0.0,
            reasoning="No suitable tasks found for the available time",
            context={"available_time": available_time, "total_tasks": len(app_ctx.tasks)}
        )

    # Score tasks based on multiple factors
    best_task = None
    best_score = 0.0
    best_reasoning = ""

    for task in available_tasks:
        score = 0.0
        reasoning_parts = []

        # Priority scoring (higher priority = higher score)
        priority_score = (6 - task.priority) * 20  # Critical=100, High=80, etc.
        score += priority_score
        reasoning_parts.append(f"Priority: {priority_score}")

        # Time fit scoring (tasks that fit well in available time)
        time_ratio = task.time_cost / available_time
        if 0.5 <= time_ratio <= 1.0:
            time_score = 30  # Perfect fit
        elif time_ratio < 0.5:
            time_score = 20  # Quick task
        else:
            time_score = 10  # Might take longer
        score += time_score
        reasoning_parts.append(f"Time fit: {time_score}")

        # Deadline urgency
        if task.due_date:
            days_until_due = (task.due_date - datetime.now()).days
            if days_until_due <= 1:
                urgency_score = 50
            elif days_until_due <= 3:
                urgency_score = 30
            elif days_until_due <= 7:
                urgency_score = 15
            else:
                urgency_score = 5
            score += urgency_score
            reasoning_parts.append(f"Urgency: {urgency_score}")

        # Benefit/importance scoring
        benefit_score = task.benefit * 2
        importance_score = task.importance * 3
        score += benefit_score + importance_score
        reasoning_parts.append(f"Value: {benefit_score + importance_score}")

        if score > best_score:
            best_score = score
            best_task = task
            best_reasoning = "; ".join(reasoning_parts)

    if best_task:
        return TaskRecommendation(
            task_id=best_task.id,
            score=best_score,
            reasoning=f"Recommended '{best_task.name}' - {best_reasoning}",
            context={
                "available_time": available_time,
                "task_time_cost": best_task.time_cost,
                "task_priority": best_task.priority,
                "current_context": current_context
            }
        )

    return TaskRecommendation(
        task_id="",
        score=0.0,
        reasoning="No suitable tasks found",
        context={"available_time": available_time}
    )


# Resources
@mcp.resource("task://{task_id}")
def get_task_details(task_id: str, ctx: Context) -> str:
    """Get detailed information about a specific task"""
    app_ctx = ctx.request_context.lifespan_context

    task = app_ctx.tasks.get(task_id)
    if not task:
        return f"Task {task_id} not found"

    return f"""
# Task: {task.name}

**ID:** {task.id}
**Status:** {task.status.value}
**Priority:** {TaskPriority(task.priority).name}
**Progress:** {task.progress * 100:.1f}%

## Description
{task.description}

## Details
- **Time Cost:** {task.time_cost} hours
- **Actual Time Spent:** {task.actual_time_spent} hours
- **Importance:** {task.importance}/5
- **Benefit:** {task.benefit}/10
- **Marginal Cost:** {task.marginal_cost}/5
- **Due Date:** {task.due_date.isoformat() if task.due_date else 'Not set'}
- **Created:** {task.created_at.isoformat()}
- **Updated:** {task.updated_at.isoformat()}

## Dependencies
{', '.join(task.dependencies) if task.dependencies else 'None'}

## Subtasks
{', '.join(task.subtasks) if task.subtasks else 'None'}
"""


@mcp.resource("project://{project_id}/summary")
def get_project_summary(project_id: str, ctx: Context) -> str:
    """Get a comprehensive summary of a project and its tasks"""
    app_ctx = ctx.request_context.lifespan_context

    project = app_ctx.projects.get(project_id)
    if not project:
        return f"Project {project_id} not found"

    # Get project tasks
    project_tasks = [t for t in app_ctx.tasks.values() if t.project_id == project_id]

    # Calculate statistics
    total_tasks = len(project_tasks)
    completed_tasks = len([t for t in project_tasks if t.status == TaskStatus.COMPLETED])
    in_progress_tasks = len([t for t in project_tasks if t.status == TaskStatus.IN_PROGRESS])
    total_time_estimated = sum(t.time_cost for t in project_tasks)
    total_time_spent = sum(t.actual_time_spent for t in project_tasks)

    completion_rate = (completed_tasks / total_tasks * 100) if total_tasks > 0 else 0

    return f"""
# Project Summary: {project.name}

**ID:** {project.id}
**Status:** {project.status}
**Created:** {project.created_at.isoformat()}

## Description
{project.description}

## Progress Overview
- **Total Tasks:** {total_tasks}
- **Completed:** {completed_tasks} ({completion_rate:.1f}%)
- **In Progress:** {in_progress_tasks}
- **Pending:** {total_tasks - completed_tasks - in_progress_tasks}

## Time Tracking
- **Estimated Total Time:** {total_time_estimated} hours
- **Actual Time Spent:** {total_time_spent} hours
- **Efficiency:** {(total_time_estimated / total_time_spent * 100) if total_time_spent > 0 else 0:.1f}%

## Task Breakdown by Priority
{_get_priority_breakdown(project_tasks)}

## Upcoming Deadlines
{_get_upcoming_deadlines(project_tasks)}
"""


def _get_priority_breakdown(tasks: List[Task]) -> str:
    """Helper function to get priority breakdown"""
    priority_counts = {}
    for task in tasks:
        priority_name = TaskPriority(task.priority).name
        priority_counts[priority_name] = priority_counts.get(priority_name, 0) + 1

    breakdown = []
    for priority, count in priority_counts.items():
        breakdown.append(f"- **{priority}:** {count}")

    return '\n'.join(breakdown) if breakdown else "No tasks"


def _get_upcoming_deadlines(tasks: List[Task]) -> str:
    """Helper function to get upcoming deadlines"""
    upcoming = [t for t in tasks if t.due_date and t.due_date > datetime.now() and t.status != TaskStatus.COMPLETED]
    upcoming.sort(key=lambda t: t.due_date)

    if not upcoming:
        return "No upcoming deadlines"

    deadline_list = []
    for task in upcoming[:5]:  # Show next 5 deadlines
        days_until = (task.due_date - datetime.now()).days
        deadline_list.append(f"- **{task.name}:** {task.due_date.strftime('%Y-%m-%d')} ({days_until} days)")

    return '\n'.join(deadline_list)


# Prompts
@mcp.prompt()
def task_review_prompt(task_id: str, ctx: Context) -> str:
    """Generate a prompt for reviewing and analyzing a completed task"""
    app_ctx = ctx.request_context.lifespan_context

    task = app_ctx.tasks.get(task_id)
    if not task:
        return f"Task {task_id} not found"

    return f"""Please conduct a comprehensive review of the following completed task:

**Task:** {task.name}
**Description:** {task.description}
**Original Estimate:** {task.time_cost} hours
**Actual Time Spent:** {task.actual_time_spent} hours
**Priority:** {TaskPriority(task.priority).name}
**Completion Date:** {task.updated_at.isoformat()}

Please analyze:
1. **Time Accuracy:** Was the original estimate accurate? What factors contributed to any variance?
2. **Quality Assessment:** How well was the task completed? Were all requirements met?
3. **Process Efficiency:** What went well during execution? What could be improved?
4. **Lessons Learned:** What insights can be applied to similar future tasks?
5. **Recommendations:** What would you do differently next time?

Provide specific, actionable insights that can improve future task planning and execution."""


@mcp.prompt()
def risk_mitigation_prompt(task_id: str, ctx: Context) -> str:
    """Generate a prompt for developing risk mitigation strategies"""
    app_ctx = ctx.request_context.lifespan_context

    task = app_ctx.tasks.get(task_id)
    if not task:
        return f"Task {task_id} not found"

    # Get risk assessment
    risk_assessment = predict_task_risks(task_id, ctx)

    return f"""Develop comprehensive risk mitigation strategies for the following task:

**Task:** {task.name}
**Description:** {task.description}
**Risk Level:** {risk_assessment.risk_level}/5
**Risk Probability:** {risk_assessment.probability:.1%}

**Identified Risk Factors:**
{chr(10).join(f'- {factor}' for factor in risk_assessment.risk_factors)}

**Current Mitigation Strategies:**
{chr(10).join(f'- {strategy}' for strategy in risk_assessment.mitigation_strategies)}

Please provide:
1. **Additional Risk Factors:** Are there any risks we haven't considered?
2. **Enhanced Mitigation Strategies:** Improve upon the current strategies
3. **Contingency Plans:** What should we do if risks materialize?
4. **Early Warning Indicators:** How can we detect risks before they become problems?
5. **Resource Requirements:** What additional resources might be needed for risk mitigation?

Focus on practical, implementable solutions that can be integrated into the task execution plan."""


def create_server() -> FastMCP:
    """Create and return the configured MCP server"""
    return mcp


if __name__ == "__main__":
    # Run the server
    mcp.run()
